function [h,hvec,score]=hbcv(A,k,steps)

% HBCV        Biased cross-validation estimate of h.
%
%             Examples: load kddata;
%
%             (1) hbcv(suicide,'knorm');
%
%             (2) [h,hvec,score]=hbcv(maddiam,'knorm'); 
%                 plot(hvec,score)     
%
%             HBCV(A,K,STEPS)
%
%             Christian C. Beardah 1995 

n=length(A);

if nargin==2,
  steps=100;
end;

H=hns(A,k);

hvec=linspace(0.01,2*H,steps);

M=A*ones(size(A'));

Y1=(M-M');
 
for i=1:steps,

  sig=sqrt(2)*hvec(i);

  Y=Y1/sig;

  term2=(Y.^4-6*Y.^2+3).*exp(-0.5*Y.^2)/sqrt(2*pi);

  term2=term2-diag(diag(term2));

  term2=sum(sum(term2))/sig^5;

  score(i)=1/(n*hvec(i)*2*sqrt(pi))+hvec(i)^4*term2/(4*n^2);

end;

[L,I]=min(score);

h=hvec(I);
