% KDEDEMO1    Demonstrate UIControls with 1D Kernel Density Estimates.
%
%             Example: data=randn(500,1); (optional - defines data)
%                      kdedemo1    
%
%             Menus: (a) DATA - A choice of five 1D data sets.
%
%             Pop-ups: (a) KERNEL - A choice of five well-known kernel functions.
%                      (b) HSELECT - A choice of five data-based h selection strategies.
%
%             Check-boxes: (a) ADAPTIVE - Turns adaptive method on/off.
%                          (b) HOLD - Holds the current plot allowing multiple
%                                     plots on the same axes.
%
%             Push buttons: (a) END - Quit demonstration,
%                           (b) HELP - Output these comments.
%
%             Editable text: (a) H - Click here to enter a value for H.  Values 
%                                of H=0 cause the routine to use an 'optimal' 
%                                value.
% 
%             Christian C. Beardah 1995  

echo off

% Initialize

load kddata;

if exist('data')==0,
  data=lukesh1(:,1);
  import=0;
  else,
    data=data(:,1);
    import=1;
end;

col='y';
sym=['- ';'--';'-.';': '];

fix=0;
kernel='knorm';
count=1;
sstr='-';
pstr=[col,sstr];
mesh=512;
adapt=0;
h=hns(data,kernel);

figure('Name','Uni-variate KDE demo');

black=[0 0 0];
set(gcf,'color',black');
gcfpos = zeros(1,4);

[x,y]=kdfft1(data,kernel,mesh,h,0);
plot(x,y,'y-');

% Callback strings

helps = 'help kdedemo1;';

ends = 'close; clear global; clear; load kddata;';

fixhs = ['if get(fixhh,''value'')==1, fix=1; else, fix=0; end;'];

replaces = ['if get(replaceh,''value'')==1, rep=1; else, rep=0; end;'];

canons = ['if get(canonh,''value'')==1, kernel(1)=''c''; else, kernel(1)=''k''; end; '];

calcs = ['if adapt==0, [x,y]=kdfft1(data,kernel,mesh,h,0);',...
        'else, [x,z,y,h,hl]=kdplot(data,kernel,mesh,adapt,h,0); end;',...
        'pstr=[col,sstr]; plot(x,y,pstr);'];

plotbumps = ['n=length(data); hold on; for i=1:n, if adapt==0,',...
             'y=feval(kernel,(x-data(i))/h)/(n*h); else,',...
             'y=feval(kernel,(x-data(i))/hl(i))/(n*hl(i)); end; plot(x,y,''w--''); end;'];

plothists = ['n=length(data); plot(x,y,pstr); hold on; bins=str2num(get(binsh,''string''));',...
             '[hn,hx]=hist(data,bins); hn=hn/((hx(2)-hx(1))*n); bar(hx,hn,''w--'');',...
             'if get(histh,''value'')==0, set(histh,''value'',1); end; hold off;'];

plots = [canons, calcs, 'if ishold, count=count+1; end;',...
         'v=get(bumpsh,''value''); if v==1,', plotbumps, 'hold off; end;',...
         'v=get(histh,''value''); if v==1,', plothists, 'hold off; end;'];

hvals = ['h=str2num(get(hvalh,''string'')); if h<0, disp(''ERROR - negative h''); else,',plots,'end;'];

hoffs = ['hold off; set(holdh,''value'',0); count=1;'];

holds = ['if get(holdh,''value'')==1, hold on; count=2; else,', hoffs ,'end;'];

bumps = ['v=get(bumpsh,''value''); if v==1,', plotbumps, 'else,', plots ,'end; hold off;'];

hists = ['v=get(histh,''value''); if v==1,', plothists, 'else,', plots ,'end; hold off;'];

adapts = ['if get(adapth,''value'')==1, adapt=1; else, adapt=0; end; ',hvals;];

hselects = ['v=get(hselecth,''value''); if v==1, h=hns(data,kernel);',...
           'elseif v==2 & kernel==''knorm'', h=hdpi(data,kernel,1);',...
           'elseif v==3 & kernel==''knorm'', h=hdpi(data,kernel,2);',...
           'elseif v==4 & kernel==''knorm'', h=hdpi(data,kernel,3);',...
           'elseif v==5 & kernel==''knorm'', h=hste(data,kernel);',...
           'elseif v==6, h=hstt(data,kernel);',...
           'elseif v==7 & kernel==''knorm'', h=hlscv(data,kernel);',...
           'elseif v==8 & kernel==''knorm'', h=hbcv(data,kernel);',...
           'elseif v==9 & kernel==''knorm'', h=hboot(data,kernel);',...
           'elseif v==10 & kernel==''knorm'', h=hscv(data,kernel);',...
           'else, h=hns(data,kernel); set(hselecth,''value'',1); end;',...
           'set(hvalh,''string'',num2str(h));', plots];

ssvals = [replaces,'ss=str2num(get(ssvalh,''string'')); set(ssvalh,''string'',num2str(ss)),',...
          'if ss<=0 | ss>length(data), disp(''ERROR - illegal sample size''); else,',...
          'temp=data; data=sample(data,ss,rep);',fixhs,'if fix==0,',hselects,'else,',plots,'end; data=temp; end;'];

kerpops = ['v=get(kerpoph,''value''); if v==1, kernel=''knorm'';',...
           'elseif v==2, kernel=''kepan'';',...
           'elseif v==3, kernel=''krect'';',...
           'elseif v==4, kernel=''ktria'';',...
           'elseif v==5, kernel=''kbiwe'';',...
           'elseif v==6, kernel=''ktriw'';',...
           'elseif v==7, kernel=''klapl'';',...
           'elseif v==8, kernel=''klogi'';',...
           'end;',fixhs,'if fix==0,',hselects,'else,',plots,'end;'];

% Frames

frame1h = uicontrol('style','frame','units','normal','pos',[.84 .73 .15 .2]);

frame2h = uicontrol('style','frame','units','normal','pos',[.84 .53 .15 .13]);

frame3h = uicontrol('style','frame','units','normal','pos',[.84 .39 .15 .13]);

% Check boxes

adapth = uicontrol('style','checkbox','units','normal','pos',[.85 .86 .13 .05],...
  'string','Adaptive','call',adapts);

holdh = uicontrol('style','checkbox','units','normal','pos',[.85 .74 .09 .05],...
  'string','Hold','call',holds);

canonh = uicontrol('style','checkbox','units','normal','pos',[.75 .02 .235 .035],...
  'string','Canonical kernels','call',canons);

fixhh = uicontrol('style','checkbox','units','normal','pos',[.015 .02 .15 .035],...
  'string','Fix h','call',fixhs);

bumpsh = uicontrol('style','checkbox','units','normal','pos',[.2 .02 .15 .035],...
  'string','Add bumps','call',bumps);

replaceh = uicontrol('style','checkbox','units','normal','pos',[.85 .54 .13 .05],...
  'string','Replace','call',replaces);

histh = uicontrol('style','checkbox','units','normal','pos',[.85 .4 .13 .05],...
  'string','Hist.','call',hists);

% Pop-up menus

kerpoph = uicontrol('style','popupmenu','units','normal','pos',[.8 .94 .19 .08],...
  'string','Normal|Epanechnikov|Rectangular|Triangular|Bi-weight|Triweight|Laplace|Logistic','call',kerpops);   

hselecth = uicontrol('style','popupmenu','units','normal','pos',[.8 .67 .19 .08],...
  'string','Normal Scale|DPI,1|DPI,2|DPI,3|STE,2|STT|LSCV|BCV|BootstrapCV|SCV','call',hselects);   

datamenus = ['set(suich,''checked'',''off''),',...
             'set(geysh,''checked'',''off''),',...
             'set(snowh,''checked'',''off''),',...
             'set(pinsh,''checked'',''off''),',...
             'set(cupdh,''checked'',''off''),',...
             'set(glass5h,''checked'',''off''),',...
             'set(glass8h,''checked'',''off''),',...
             'set(n01h,''checked'',''off''),',...
             'set(nmdh,''checked'',''off''),',...
             'set(smallh,''checked'',''off''),',...
             'set(diamh,''checked'',''off'');'];

% Menus

datamenuh = uimenu(gcf,'label','Data');

suich = uimenu(datamenuh,'label','Suicide Time (86)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=suicide; set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(suich,''checked'',''on'')']);
geysh = uimenu(datamenuh,'label','Old Faithful (107)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=geyser; set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(geysh,''checked'',''on'')']);
snowh = uimenu(datamenuh,'label','Buffalo Snowfall (63)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=snowfall; set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(snowh,''checked'',''on'')']);
diamh = uimenu(datamenuh,'label','Pot Diameters (81)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=maddiam; set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(diamh,''checked'',''on'')']);
pinsh = uimenu(datamenuh,'label','Hairpin Lengths (224)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=pins10(:,1); set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(pinsh,''checked'',''on'')']);
cupdh = uimenu(datamenuh,'label','Cup Diameters (60)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=lukesh1(:,1); set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(cupdh,''checked'',''on'')']);
glass5h = uimenu(datamenuh,'label','Nickel (361)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=glass(:,5); set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(glass5h,''checked'',''on'')']);
glass8h = uimenu(datamenuh,'label','Manganese (361)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=glass(:,8); set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(glass8h,''checked'',''on'')']);
n01h = uimenu(datamenuh,'label','N(0,1) (500)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=randn(500,1); set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(n01h,''checked'',''on'')']);
nmdh = uimenu(datamenuh,'label','NMD (500)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h));',...
   'data=snmd(500,[0.75 0.25]'',[1 1/3]'',[0 3/2]'')''; set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(nmdh,''checked'',''on'')']);
smallh = uimenu(datamenuh,'label','Small (7)','callback',...
  [hoffs,'set(hvalh,''string'',num2str(h)); data=small; set(ssvalh,''string'',num2str(length(data)));', hselects,...
   'if get(fixhh,''value'')==1, set(fixhh,''value'',0); end;',...
   'if get(canonh,''value'')==1, set(canonh,''value'',0); end;', datamenus,...
   'set(smallh,''checked'',''on'')']);

sstrmenus = ['set(solidh,''checked'',''off''),',...
             'set(dashh,''checked'',''off''),',...
             'set(dotdashh,''checked'',''off''),',...
             'set(doth,''checked'',''off'');'];

sstrmenuh = uimenu(gcf,'label','Print style');

solidh = uimenu(sstrmenuh,'label','Solid','callback',...
  ['sstr=''-'';', sstrmenus, 'set(solidh,''checked'',''on'')']);
dashh = uimenu(sstrmenuh,'label','Dashed','callback',...
  ['sstr=''--'';', sstrmenus, 'set(dashh,''checked'',''on'')']);
dotdashh = uimenu(sstrmenuh,'label','Dash-dotted','callback',...
  ['sstr=''-.'';', sstrmenus, 'set(dotdashh,''checked'',''on'')']);
doth = uimenu(sstrmenuh,'label','Dotted','callback',...
  ['sstr='':'';', sstrmenus, 'set(doth,''checked'',''on'')']);

set(solidh,'checked','on');

colmenus = ['set(yellowh,''checked'',''off''),',...
            'set(redh,''checked'',''off''),',...
            'set(whiteh,''checked'',''off''),',...
            'set(blueh,''checked'',''off''),',...
            'set(magentah,''checked'',''off''),',...
            'set(cyanh,''checked'',''off''),',...
            'set(greenh,''checked'',''off'');'];

colmenuh = uimenu(gcf,'label','Colour');

yellowh = uimenu(colmenuh,'label','Yellow','callback',...
  ['col=''y'';', colmenus, 'set(yellowh,''checked'',''on'')']);
redh = uimenu(colmenuh,'label','Red','callback',...
  ['col=''r'';', colmenus, 'set(redh,''checked'',''on'')']);
whiteh = uimenu(colmenuh,'label','White','callback',...
  ['col=''w'';', colmenus, 'set(whiteh,''checked'',''on'')']);
blueh = uimenu(colmenuh,'label','Blue','callback',...
  ['col=''b'';', colmenus, 'set(blueh,''checked'',''on'')']);
magentah = uimenu(colmenuh,'label','Magenta','callback',...
  ['col=''m'';', colmenus, 'set(magentah,''checked'',''on'')']);
cyanh = uimenu(colmenuh,'label','Cyan','callback',...
  ['col=''c'';', colmenus, 'set(cyanh,''checked'',''on'')']);
greenh = uimenu(colmenuh,'label','Green','callback',...
  ['col=''g'';', colmenus, 'set(greenh,''checked'',''on'')']);

set(yellowh,'checked','on');

if import==0,
  set(cupdh,'checked','on');
end;

% Push buttons

endh = uicontrol('style','push','units','normal','pos',[.91 .19 .08 .06],...
        'string','End','call',ends);
helph = uicontrol('style','push','units','normal','pos',[.91 .11 .08 .06],...
        'string','Help','call',helps);

% Editable text

hvalh = uicontrol('style','edit','units','normal','pos',[.91 .8 .05 .04],...
        'string',num2str(h),'min',0,'call',hvals);
hvalt = uicontrol('style','text','units','normal','pos',[.85 .8 .05 .04],...
     'string','h:');
ssvalh = uicontrol('style','edit','units','normal','pos',[.93 .6 .05 .04],...
     'string',num2str(length(data)),'min',1,'call',ssvals);
ssvalt = uicontrol('style','text','units','normal','pos',[.85 .6 .07 .04],...
     'string','Ssize:');
binsh = uicontrol('style','edit','units','normal','pos',[.93 .46 .05 .04],...
     'string','10','min',1,'call',plots);
binst = uicontrol('style','text','units','normal','pos',[.85 .46 .07 .04],...
     'string','Bins:');