function pclvl3a(p,A,X,Y,Z,dgrid,ddata,sstr)

% PCLVL3A     Kernel Density estimate p% contour plot for tri-variate data.
%
%             Example: 
%
%             PCLVL3A(P,A,X,Y,Z,DGRID,DDATA) displays a surface which encloses P% 
%             of the data.  Groups are formed by ranking the data by 
%             kernel density estimate.
%
%             The rows of A contain the whole data-set.
% 
%             The kernel is given by the function in K.
%             'KMVN' is the multivariate normal distribution.
%
%             INC controls the granularity of the mesh.
%
%             The optional parameter H represents the window width.
%             If omitted, an 'optimal' value is used.
%
%             Christian C. Beardah 1996

[rX,ngrid]=size(X);

xr=(max(max(X))-min(min(X)))/ngrid;
yr=(max(max(Y))-min(min(Y)))/ngrid;
zr=(max(max(Z))-min(min(Z)))/ngrid;

n=length(ddata);

x=A(:,1);
y=A(:,2);
z=A(:,3);

w=zeros(length(p),2);

p=100-p;
p=sort(p);
p=p(length(p):-1:1);

[order index]=sort(ddata);

m=ceil(n*p/100);

if length(p)>1,
  col=['wbyrcm'];
  else,
    col=sstr;
end;

for i=1:length(p),
  if m(i)==0,
    v(i)=0.9*order(1); 
  end;

  if m(i)>0 & m(i)<n,
    v(i)=(order(m(i))+order(m(i)+1))/2;
  end;

  if m(i)==n,
    v(i)=(order(n-1)+order(n))/2;
  end;

  if i==1,
    dtest=(dgrid>v(i));
    else,
      dtest=(dgrid>v(i))&(dgrid<v(i-1));
  end;
  [ip,jp]=find(dtest);
  for k=1:length(ip),
    hpt(k,:)=[X(ip(k),jp(k)) Y(ip(k),jp(k)) Z(ip(k),jp(k))];
  end;

  if i==length(p),
    xr=0; yr=0; zr=0;
  end;

  randx=xr*rand(length(ip),1)/(i+1);
  randy=yr*rand(length(ip),1)/(i+1);
  randz=zr*rand(length(ip),1)/(i+1);

  plot3(hpt(1:length(ip),1)+randx,hpt(1:length(ip),2)+randy,hpt(1:length(ip),3)+randz,[col(i),'.']);

  if i==1,
    hold on;
  end;

end;

xlabel('x values');
ylabel('y values');
zlabel('z values');

hold off;